# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'integrationPJ.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::CreatePJTask, type: :helper do
  let!(:xsd) {Nokogiri::XML::Schema(File.read(schema_path))}
  let!(:user1) {create(:user)}
  
  
  context 'with simple housing_offer' do
    let!(:applicant1) {
      create(
        :applicant,
        user: user1,
        monthly_charges: {},
        nationality: "1"
      )
    }
    
    let!(:housing_request1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant1
      )
    end
    let!(:nud1) do
      create(
        :nud,
        housing_request: housing_request1,
        department: '12',
        creation_date: '2016-23-14T14:53:22',
        unique_number: "1110320019869G1175"
      )
    end
  end
  
  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end
  
  context 'with simple housing_request' do
    let!(:applicant1) {
      create(
        :applicant,
        user: user1,
        monthly_charges: {},
        nationality: "1"
      )
    }
    let!(:applicant2) {
      create(
        :applicant,
        user: user1,
        monthly_charges: {},
        nationality: "1"
      )
    }
    
    let!(:housing_request1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant1,
        coapplicants: [
          {rel_type: 'Conjoint (e)', id: applicant2.id}
        ]
      )
    end
    let!(:nud1) do
      create(
        :nud,
        housing_request: housing_request1,
        department: '12',
        creation_date: '2016-23-14T14:53:22',
        unique_number: "1110320019869G1175"
      )
    end
    let!(:create_pj_task1) do
      required_document = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
      file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
      blob = file.read
      required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
      required_document.save
      build(
        :create_pj_task,
        options: {housing_request_id: housing_request1.id,
                  num_unique: nud1.unique_number,
                  required_document_id: required_document.id}
      )
    end
    
    describe 'using to_xml' do
      it 'should return a valid xml' do
        expect(xsd.validate(Nokogiri::XML(create_pj_task1.to_xml))).to eq([])
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        VCR.use_cassette("createPJ", :match_requests_on => [:method, :uri, :path]) do
          required_document.create_file_in_sne(nud1.unique_number, housing_request1.id)
          expect(required_document.reload.sne_piece_id).not_to be(nil)
        end
      
      end
    end
    describe 'using to_xml' do
      it 'should return a valid xml' do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        required_document.sne_piece_id = "16829"
        required_document.save
        
        VCR.use_cassette("updatePJ", :match_requests_on => [:method, :uri, :path]) do
          so = required_document.update_file_in_sne(nud1.unique_number, housing_request1.id)
          ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml, "put", "Hermes::RequiredDocument", required_document.id.to_s)
  
        end
      end
    end
    describe "syncro_files_from_sne" do
      it "do tests" do
        hrf = housing_request1.reload.housing_request_folder
        VCR.use_cassette("syncro_files_from_sne", :match_requests_on => [:method, :uri, :path]) do
           file_list = hrf.get_file_list("1110320019869G1175")

        end
      end
    end
    describe "get_file_from_sne" do
      it "do tests" do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        
        initial_doc_id = required_document.doc_id
        hrf = housing_request1.reload.housing_request_folder
        file = "<informationPJ>
        <auteurDepot>Guichet test AL Services</auteurDepot>
        <categorieCOD>SITPRO</categorieCOD>
        <categorieLIB>Situation professionnelle</categorieLIB>
        <creationAuteur>Guichet test AL Services</creationAuteur>
        <creationDAT>2020-03-30T13:51:35.000+02:00</creationDAT>
        <dateDepot>2020-03-30T13:51:35.000+02:00</dateDepot>
        <derniereActionAuteur>Guichet test AL Services</derniereActionAuteur>
        <derniereActionDAT>2020-03-30T13:51:35.000+02:00</derniereActionDAT>
        <derniereActionType>Création</derniereActionType>
        <documentInterne>false</documentInterne>
        <fonctionnelleDAT>2034-02-13T00:00:00.000+01:00</fonctionnelleDAT>
        <idFichier>16827</idFichier>
        <idPiece>16827</idPiece>
        <indicePertinence>Non vérifié</indicePertinence>
        <nomFichier>SNE000190204000005.pdf</nomFichier>
        <numeroUnique>1110320019869G1175</numeroUnique>
        <sensibilite>false</sensibilite>
        <type>Carte d'étudiant</type>
    </informationPJ>"
        existing_doc = Hash.from_xml(file.gsub("\n", ''))
        VCR.use_cassette("get_file_from_sne", :match_requests_on => [:method, :uri, :path]) do
          
          required_document.get_file_from_sne(existing_doc, "1110320019869G1175")
          required_document.reload
          expect(required_document.doc_id == initial_doc_id).to eq(false)
        end
      end
      
    end
    describe "delete_file_from_sne" do
      it "do tests" do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        required_document.sne_piece_id = "16829"
        required_document.save
       
        VCR.use_cassette("delete_file_from_sne", :match_requests_on => [:method, :uri, :path]) do
        so =   required_document.delete_file_in_sne("1110320019869G1175", housing_request1.id.to_s)
        end
      end

    end
  end
end
